#include "simulation.h"
#include "random.h"
#include <time.h>
#include <stdlib.h>

//-------------------------------SHO demo-------------------------------- 
class TService;

CHead     *Queue;                              
TService  *Service;                              


class TRequest : public CProcess                 // request in the queueing system
{                                                
  virtual void Run();                            // request's behaviour
  
};


class TService : public CProcess                 // process servicing requests
{                                                 
  virtual void Run();                            // service's behaviour
};

void TRequest::Run()                           
{
  printf("Incoming %f\n",Time ());
  (new TRequest)->ActivateDelay(70);               // scheduling incoming of next request
  if (Queue->Empty()) {                            // if the queue is empty 
    Into(Queue);                                  // inserting request into queue
    printf("Waking up service ....\n");             
    Service->ActivateAt(Time ());                  // waking up service process
  } else {
    printf("There is/are %d request(s) in the queue, ranking ...\n",Queue->Cardinal());
    Into(Queue);                                  // ranking the request process to the end of the queue
  }
  printf("Going to sleep ... \n");
  Passivate();                                      // waiting for the finishing the service
  printf("Leaving system %f\n",Time ());
}

void TService::Run()                               
{
  CProcess* P;                                       // temp. variable
  while(true) {
     if (Queue->Empty()) {
       printf("Queue is empty, going to sleep ... \n");
       Passivate();                                 // service is waiting for incoming requests
     } else {
       printf("Begin of service in %f\n",Time ());     // in the queue, there is at least one request
       P=(CProcess*)Queue->First();                
       P->Out();                                      // removing the request from the queue
       Hold(negexp(0.02));                             // servicing time (mean value 50 )
       P->ActivateAt (Time());                        // executing request
       printf("End of service in %f\n",Time ());
     }
  }
}
/*----------------------------------------------*/
void CSimulation::Run()                          // main process
{

  if (argc > 1) {
	int i;
	printf("Calling with %d parameters:\n", argc-1);
	for (i=1; i<argc; i++) printf("\t%s\n", (*argv)[i]);
  }

  printf("--- Demonstration of SIMULATION module -- time %f\n", Time());

  srand( (unsigned)time( NULL ) );               // inicialization on random number generator
  Service = new TService();                      // creating service process
  Queue  = new CHead();                          // creating instance of queue
  (new TRequest)->ActivateAt(Time ());           // creating and scheduling 1st request
  
  Hold(500);                                     // waiting before simulation end
  printf("Queue length: %ld\n\n", Queue -> Cardinal());
  printf("End of simulation --- time: %f\n\n", Time ());
  exit(0);
};
